package it.innove;

import static com.facebook.react.bridge.UiThreadUtil.runOnUiThread;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.util.Log;

import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.Callback;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReadableArray;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.WritableMap;

public class LegacyScanManager extends ScanManager {

    public LegacyScanManager(ReactApplicationContext reactContext, BleManager bleManager) {
        super(reactContext, bleManager);
    }

    @Override
    public void stopScan(Callback callback) {
        // update scanSessionId to prevent stopping next scan by running timeout thread
        scanSessionId.incrementAndGet();

        getBluetoothAdapter().stopLeScan(mLeScanCallback);
        callback.invoke();
    }

    private BluetoothAdapter.LeScanCallback mLeScanCallback =
            new BluetoothAdapter.LeScanCallback() {

                @Override
                public void onLeScan(final BluetoothDevice device, final int rssi,
                                     final byte[] scanRecord) {
                    runOnUiThread(new Runnable() {
                        @Override
                        public void run() {
                            Log.i(BleManager.LOG_TAG, "DiscoverPeripheral: " + device.getName());

                            Peripheral peripheral = bleManager.getPeripheral(device);
                            if (peripheral == null) {
                                peripheral = new Peripheral(device, rssi, scanRecord, bleManager.getReactContext());
                            } else {
                                peripheral.updateData(scanRecord);
                                peripheral.updateRssi(rssi);
                            }
                            bleManager.savePeripheral(peripheral);

                            WritableMap map = peripheral.asWritableMap();
                            bleManager.sendEvent("BleManagerDiscoverPeripheral", map);
                        }
                    });
                }


            };

    @Override
    public void scan(ReadableArray serviceUUIDs, final int scanSeconds, ReadableMap options, Callback callback) {
        if (serviceUUIDs.size() > 0) {
            Log.d(BleManager.LOG_TAG, "Filter is not working in pre-lollipop devices");
        }
        getBluetoothAdapter().startLeScan(mLeScanCallback);

        if (scanSeconds > 0) {
            Thread thread = new Thread() {
                private int currentScanSession = scanSessionId.incrementAndGet();

                @Override
                public void run() {

                    try {
                        Thread.sleep(scanSeconds * 1000);
                    } catch (InterruptedException ignored) {
                    }

                    runOnUiThread(new Runnable() {
                        @Override
                        public void run() {
                            BluetoothAdapter btAdapter = getBluetoothAdapter();
                            // check current scan session was not stopped
                            if (scanSessionId.intValue() == currentScanSession) {
                                if (btAdapter.getState() == BluetoothAdapter.STATE_ON) {
                                    btAdapter.stopLeScan(mLeScanCallback);
                                }
                                WritableMap map = Arguments.createMap();
                                map.putInt("status", 0);
                                bleManager.sendEvent("BleManagerStopScan", map);
                            }
                        }
                    });

                }

            };
            thread.start();
        }
        callback.invoke();
    }

    @Override
    public boolean isScanning() {
        return false;
    }
}
